@@ -31,13 +31,13 @@ class Database:
                        end_date_str + '&' + freq_str + '&' + price_adj_str + '&' + vol_adj_str
             
             response = requests.get(link_str)
-            
+
             if price_adj == False:
                 ohlc_result = json.loads(json.loads(response.content).replace("'", "\""))
             else:
                 result = ast.literal_eval(json.loads(response.content))
                 ohlc_result = result[0]
-                qfq_result  = result[1]
+                qfq_result = result[1]
                 qfq_df = pd.DataFrame(qfq_result, columns=['date', 'qfq_factor'])
 
             df_list = []
@@ -45,31 +45,34 @@ class Database:
             for item in ohlc_result:
                 date_int = item['_id']
                 df = pd.DataFrame(item['content'])
-                cols = ['datetime', 'date'] + list(df.columns)
 
                 if len(df) > 0:
-                    if 'T' in freq:
-                        df['time'] = df['time'].astype(str)
-                        df['date'] = str(date_int)
-                        df['datetime'] = df['date'] + ' ' + df['time']
-                        df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
-
-                    else:
-                        df['date'] = str(date_int)
-                        df['datetime'] = df['date']
-                        df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d')
-
-                    df = df[cols]
-                    df = df.set_index(keys='datetime')
+                    df['date'] = str(date_int)
                     df_list.append(df)
-                    
+
             df = pd.concat(df_list)
-            
+
+            if 'T' in freq:
+                df['time'] = df['time'].astype(str)
+                df['datetime'] = df['date'] + ' ' + df['time']
+                df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
+                cols = ['datetime', 'date', 'time', 'open', 'high', 'low', 'close', 'volume']
+
+            else:
+                df['datetime'] = df['date']
+                df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d')
+                cols = ['datetime', 'date', 'open', 'high', 'low', 'close', 'volume']
+
+            df = df[cols]
+
+            df = df.set_index(keys='datetime')
+            df = df.sort_index()
+
             if price_adj == True:
                 qfq_df = qfq_df[qfq_df['date'] > 20100000]
                 qfq_df = qfq_df.iloc[::-1].reset_index(drop=True)
                 qfq_df['date'] = qfq_df['date'].astype(str)
-                first_date = qfq_df.loc[qfq_df.index[0], 'date'] 
+                first_date = qfq_df.loc[qfq_df.index[0], 'date']
                 last_date = str(max([int(qfq_df.loc[qfq_df.index[-1], 'date']), int(df.loc[df.index[-1], 'date'])]))
                 qfq_df['date'] = pd.to_datetime(qfq_df['date'], format='%Y-%m-%d')
                 qfq_df = qfq_df.set_index(keys='date')
@@ -151,10 +154,137 @@ class Database:
                     err_msg += error + ','
             print(err_msg)
 
+    def get_hk_fut_ohlc(self, index, freq, start_date, end_date, rolling_day, rolling_time):
+        
+        check_bool_dict = self.check_hk_fut_ohlc_args(index, freq, start_date, end_date, rolling_day, rolling_time)
+
+        if False not in list(check_bool_dict.values()):
+
+            start_date_dt = datetime.datetime.strptime(str(start_date), '%Y%m%d').date()
+            end_date_dt = datetime.datetime.strptime(str(end_date), '%Y%m%d').date()
+
+            start_date_year = start_date_dt.year if start_date_dt.month > 1 else start_date_dt.year - 1
+            end_date_year = end_date_dt.year if end_date_dt.month < 12 else end_date_dt.year + 1
+            
+            holiday_dict = get_hk_holiday_and_expiry_date(start_date_year, end_date_year, 'datetime')
+            expiry_date_list = holiday_dict['expiry_date']
+
+            expiry_date_dict = {}
+            for expiry_date in expiry_date_list:
+                expiry_year_month_str = str(expiry_date.year - 2000) + str(expiry_date.month).zfill(2)
+                expiry_date_dict[expiry_year_month_str] = expiry_date
+
+            day_diff = (end_date_dt - start_date_dt).days
+
+            year_month_involved_list = []
+            for i in range(day_diff + 1):
+                i_date = start_date_dt + datetime.timedelta(days=i)
+                year_month = i_date.strftime('%y%m')
+                expiry_date = expiry_date_dict[year_month]
+                if i_date > expiry_date:
+                    year_month = (i_date + relativedelta(months=1)).strftime('%y%m')
+                if year_month not in year_month_involved_list:
+                    year_month_involved_list.append(year_month)
+
+            year_month_involved_list.append((datetime.datetime.strptime(year_month_involved_list[-1], '%y%m') + relativedelta(months=1)).strftime('%y%m'))
+
+            year_month_involved_list_str = json.dumps(str(year_month_involved_list))
+            year_month_involved_list_str = year_month_involved_list_str.replace('[', '')
+            year_month_involved_list_str = year_month_involved_list_str.replace(']', '')
+            year_month_involved_list_str = year_month_involved_list_str.replace('\'', '')
+            year_month_involved_list_str = year_month_involved_list_str.replace(',', '')
+            year_month_involved_list_str = year_month_involved_list_str.replace(' ', '_')
+            year_month_involved_list_str = year_month_involved_list_str.replace('\"', '')
+
+            link_url = 'http://www.hkfdb.net/data_api?'
+            token_str = 'token=' + str(self.authToken)
+            database_str = 'database=' + 'hk_fut_ohlc'
+            index_str = 'index=' + index
+            freq_str = 'freq=' + freq
+            start_date_str = 'start_date=' + str(start_date)
+            end_date_str = 'end_date=' + str(end_date)
+            rolling_day_str = 'rolling_day=' + str(rolling_day)
+            rolling_time_str = 'rolling_time=' + str(rolling_time)
+            year_month_involved_list_str = 'year_month_involved_list=' + year_month_involved_list_str
+            link_str = link_url + index_str + '&' + freq_str + '&' + rolling_day_str + '&' + rolling_time_str + '&' + year_month_involved_list_str + '&' \
+                       + token_str + '&' + database_str + '&' \
+                       + start_date_str + '&' + end_date_str
+
+            response = requests.get(link_str)
+            result_list = json.loads(json.loads(response.content).replace("'", "\""))
+
+            df_list = []
+
+            for result in result_list:
+                sub_df_list = []
+                for item in result:
+                    df = pd.DataFrame(item['content'])
+                    if len(df) > 0:
+                        sub_df_list.append(df)
+    
+                if len(sub_df_list) > 0:
+                    df = pd.concat(sub_df_list)
+                    df = df.reset_index(drop=True)
+                    date_list = list(df['date'].unique())
+                    date_list.sort()
+                    front_year_month = str(date_list[0])[2:6]
+                    back_year_month = str(date_list[-1])[2:6]
+                    front_expiry_date = int(expiry_date_dict[front_year_month].strftime('%Y%m%d'))
+                    try:
+                        back_expiry_date = int(expiry_date_dict[back_year_month].strftime('%Y%m%d'))
+                    except:
+                        print(back_year_month)
+                        print(date_list)
+                        print(expiry_date_dict)
+                        time.sleep(123456)
+
+                    for i in range(len(date_list)):
+                        date_item = date_list[i]
+    
+                        if i + rolling_day <= len(date_list) - 1:
+                            if date_list[i + rolling_day] == front_expiry_date:
+                                front_cut_off_date = date_item
+                            elif date_list[i + rolling_day] == back_expiry_date:
+                                back_cut_off_date = date_item
+                                break
+                        else:
+                            back_cut_off_date = max(date_list)
+    
+                    if df.loc[0,'expiry_date'] == '20130530':
+                        front_cut_off_date = 20130501
+
+                    df = df[(df['date'] > front_cut_off_date) | ((df['date'] == front_cut_off_date) & (df['time'] >= rolling_time))]
+                    df = df[(df['date'] < back_cut_off_date) | ((df['date'] == back_cut_off_date) & (df['time'] < rolling_time))]
+                    df_list.append(df)
+    
+            cols = ['datetime', 'date', 'time', 'open', 'high', 'low', 'close', 'volume', 'expiry_date','RTH']
+            df = pd.concat(df_list)
+    
+            df = df[df['date'] >= start_date]
+            df = df[df['date'] <= end_date]
+            df['date'] = df['date'].astype(str)
+            df['time'] = df['time'].astype(str)
+            df['time'] = df['time'].str.zfill(6)
+            df['datetime'] = df['date'] + ' ' + df['time']
+            df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
+            df['RTH'] = df['RTH'].astype(bool)
+
+            df = df[cols]
+            df = df.set_index(keys='datetime')
+            df = df.sort_index(ascending=True)
+    
+            return df
+    
+        else:
+            err_msg = 'Error in: '
+            for error in check_bool_dict:
+                if check_bool_dict[error] == False:
+                    err_msg += error + ','
+            print(err_msg)
+
     def get_hk_market_cap_hist(self, start_date, end_date):
         
-        ### should change name to check_start_end_date ###
-        check_bool_dict = self.check_start_end_date(start_date, end_date, 180)
+        check_bool_dict = self.check_start_end_date(start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
             link_url = 'http://www.hkfdb.net/data_api?'
@@ -211,8 +341,7 @@ class Database:
 
     def get_hk_buyback_by_date(self, start_date, end_date):
 
-        ### should change name to check_start_end_date ###
-        check_bool_dict = self.check_start_end_date(start_date, end_date, 2160)
+        check_bool_dict = self.check_start_end_date(start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
 
@@ -323,7 +452,6 @@ class Database:
         return date_list
     def get_us_earning_calendar_by_date(self, start_date, end_date):
         
-        ### should change name to check_start_end_date ###
         check_bool_dict = self.check_start_end_date(start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
@@ -848,7 +976,6 @@ class Database:
         freq_list = ['1T', '5T', '15T', '30T', '1D']
         freq_valid = True if freq in freq_list else False
 
-
         try:
             code_length = len(code) == 5
         except:
@@ -889,28 +1016,6 @@ class Database:
         except:
             end_after_start_date = False
         
-        try:
-            day_diff = (datetime.datetime.strptime(str(end_date),'%Y%m%d').date() - \
-                                   datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
-        except:
-            exceed_max_day_diff = False
-
-            '''
-            if freq == '1T':
-                exceed_max_day_diff = day_diff <= 36
-            elif freq == '5T':
-                exceed_max_day_diff = day_diff <= 180
-            elif freq == '15T':
-                exceed_max_day_diff = day_diff <= 540
-            elif freq == '30T':
-                exceed_max_day_diff = day_diff <= 540
-            elif freq == '30T':
-                exceed_max_day_diff = day_diff <= 1080
-            elif freq == '1D':
-                exceed_max_day_diff = day_diff <= 2160
-            '''
-        exceed_max_day_diff = True
-        
         check_bool_dict = {'freq_valid': freq_valid,
                            'code_isdigit': code_isdigit,
                            'code_length': code_length,
@@ -920,8 +1025,7 @@ class Database:
                            'end_date_is_int': end_date_is_int,
                            'end_date_length': end_date_length,
                            'end_date_future': end_date_future,
-                           'end_after_start_date': end_after_start_date,
-                           'exceed_max_day_diff': exceed_max_day_diff}
+                           'end_after_start_date': end_after_start_date}
         
         return check_bool_dict
     def check_us_stock_ohlc_args(self, code, start_date, end_date):
@@ -964,17 +1068,75 @@ class Database:
         except:
             end_after_start_date = False
         
-        '''
-        try:
-            day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
-                        datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
-            exceed_max_day_diff = day_diff <= 36
-        except:
-            exceed_max_day_diff = False
-        '''
-        exceed_max_day_diff = True
         
         check_bool_dict = {'code_length': code_length,
+                           'start_date_length': start_date_length,
+                           'start_date_is_int': start_date_is_int,
+                           'start_date_future': start_date_future,
+                           'end_date_is_int': end_date_is_int,
+                           'end_date_length': end_date_length,
+                           'end_date_future': end_date_future,
+                           'end_after_start_date': end_after_start_date}
+        
+        return check_bool_dict
+
+    def check_hk_fut_ohlc_args(self, index, freq, start_date, end_date, rolling_day, rolling_time):
+
+        freq_list = ['1T', '5T', '15T']
+        freq_valid = True if freq in freq_list else False
+
+        try:
+            index_name = (index == 'HSI') or (index == 'HHI')
+        except:
+            index_name = False
+        try:
+            start_date_is_int = isinstance(start_date, int)
+        except:
+            start_date_is_int = False
+        try:
+            start_date_length = len(str(start_date)) == 8
+        except:
+            start_date_length = False
+        try:
+            start_date_future = datetime.datetime.strptime(str(start_date),
+                                                           '%Y%m%d').date() <= datetime.datetime.now().date()
+        except:
+            start_date_future = False
+        try:
+            end_date_is_int = isinstance(end_date, int)
+        except:
+            end_date_is_int = False
+        try:
+            end_date_length = len(str(end_date)) == 8
+        except:
+            end_date_length = False
+        try:
+            end_date_future = datetime.datetime.strptime(str(end_date),
+                                                         '%Y%m%d').date() <= datetime.datetime.now().date()
+        except:
+            end_date_future = False
+        try:
+            end_after_start_date = datetime.datetime.strptime(str(end_date),
+                                                              '%Y%m%d').date() >= datetime.datetime.strptime(
+                str(start_date), '%Y%m%d').date()
+        except:
+            end_after_start_date = False
+
+        try:
+            rolling_day_int = isinstance(rolling_day, int)
+        except:
+            rolling_day_int = False
+        try:
+            rolling_day_length = rolling_day <= 5
+        except:
+            rolling_day_length = False
+        try:
+            rolling_time_int = isinstance(rolling_time, int)
+        except:
+            rolling_time_int = False
+
+        check_bool_dict = {'index_name': index_name,
+                           'freq_valid': freq_valid,
                            'start_date_length': start_date_length,
                            'start_date_is_int': start_date_is_int,
                            'start_date_future': start_date_future,
@@ -982,7 +1144,10 @@ class Database:
                            'end_date_length': end_date_length,
                            'end_date_future': end_date_future,
                            'end_after_start_date': end_after_start_date,
-                           'exceed_max_day_diff':exceed_max_day_diff}
+                           'rolling_day_int': rolling_day_int,
+                           'rolling_day_length': rolling_day_length,
+                           'rolling_time_int': rolling_time_int}
+
         return check_bool_dict
 
     def check_market_highlight_args(self, market, start_date, end_date):
@@ -1033,7 +1198,7 @@ class Database:
                            'end_after_start_date': end_after_start_date}
         return check_bool_dict
 
-    def check_start_end_date(self, start_date, end_date, max_day = 9999):
+    def check_start_end_date(self, start_date, end_date):
 
         try:
             start_date_is_int = isinstance(start_date, int)
@@ -1067,16 +1232,6 @@ class Database:
         except:
             end_after_start_date = False
         
-        '''
-        if max_day < 9999:
-            day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
-                        datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
-            exceed_max_day_diff = day_diff <= max_day
-        else:
-            exceed_max_day_diff = True
-        '''
-        exceed_max_day_diff = True
-            
         check_bool_dict = {
             'start_date_length': start_date_length,
             'start_date_is_int': start_date_is_int,
@@ -1084,8 +1239,8 @@ class Database:
             'end_date_is_int': end_date_is_int,
             'end_date_length': end_date_length,
             'end_date_future': end_date_future,
-            'end_after_start_date': end_after_start_date,
-            'exceed_max_day_diff':exceed_max_day_diff}
+            'end_after_start_date': end_after_start_date}
+        
         return check_bool_dict
 
     def check_ccass_by_id_args(self, ccass_id, start_date, end_date):
@@ -1124,15 +1279,6 @@ class Database:
         except:
             end_after_start_date = False
         
-        '''
-        try:
-            day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
-                        datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
-            exceed_max_day_diff = day_diff <= 360
-        except:
-            exceed_max_day_diff = False
-        '''
-        exceed_max_day_diff = True
         
         check_bool_dict = {'ccass_id_len' : ccass_id_len,
                            'start_date_length': start_date_length,
@@ -1141,8 +1287,7 @@ class Database:
                            'end_date_is_int': end_date_is_int,
                            'end_date_length': end_date_length,
                            'end_date_future': end_date_future,
-                           'end_after_start_date': end_after_start_date,
-                           'exceed_max_day_diff':exceed_max_day_diff}
+                           'end_after_start_date': end_after_start_date}
         return check_bool_dict
     
     def check_ccass_by_code_args(self, code, start_date, end_date):
@@ -1188,15 +1333,6 @@ class Database:
         except:
             end_after_start_date = False
         
-        '''
-        try:
-            day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
-                        datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
-            exceed_max_day_diff = day_diff <= 2160
-        except:
-            exceed_max_day_diff = False
-        '''
-        exceed_max_day_diff = True
         
         check_bool_dict = {
             'code_isdigit': code_isdigit,
@@ -1207,8 +1343,7 @@ class Database:
             'end_date_is_int': end_date_is_int,
             'end_date_length': end_date_length,
             'end_date_future': end_date_future,
-            'end_after_start_date': end_after_start_date,
-            'exceed_max_day_diff':exceed_max_day_diff}
+            'end_after_start_date': end_after_start_date}
         
         return check_bool_dict
     def check_ccass_holding_rank_args(self, code, start_date, end_date):
@@ -1255,15 +1390,6 @@ class Database:
         except:
             end_after_start_date = False
         
-        '''
-        try:
-            day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
-                        datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
-            exceed_max_day_diff = day_diff <= 360
-        except:
-            exceed_max_day_diff = False
-        '''
-        exceed_max_day_diff = True
 
         check_bool_dict = {
             'code_isdigit': code_isdigit,
@@ -1274,8 +1400,7 @@ class Database:
             'end_date_is_int': end_date_is_int,
             'end_date_length': end_date_length,
             'end_date_future': end_date_future,
-            'end_after_start_date': end_after_start_date,
-            'exceed_max_day_diff':exceed_max_day_diff}
+            'end_after_start_date': end_after_start_date}
 
         return check_bool_dict
 
@@ -1332,16 +1457,6 @@ class Database:
         except:
             end_after_start_date = False
         
-        '''
-        try:
-            day_diff = (datetime.datetime.strptime(str(end_date), '%Y%m%d').date() - \
-                        datetime.datetime.strptime(str(start_date), '%Y%m%d').date()).days
-            exceed_max_day_diff = (deri == 'opt' and day_diff <= 36) or \
-                                  (deri == 'fut' and day_diff <= 2160)
-        except:
-            exceed_max_day_diff = False
-        '''
-        exceed_max_day_diff = True
         
         check_bool_dict = {'code_length': code_length,
                            'start_date_length': start_date_length,
@@ -1351,100 +1466,100 @@ class Database:
                            'end_date_length': end_date_length,
                            'end_date_future': end_date_future,
                            'end_after_start_date': end_after_start_date,
-                           'exceed_max_day_diff': exceed_max_day_diff,
                            'deri_type': deri_type,
                            'exp_type': exp_type}
 
         return check_bool_dict
     
     
-    ################################ other tools ################################ 
+################################ other tools ################################ 
 
-    def get_holiday_from_gov(self, year):
-        r = requests.get('https://www.gov.hk/en/about/abouthk/holiday/' + year + '.htm')
-        soup = BeautifulSoup(r.content.decode('utf-8'), 'lxml')
-        items = soup.find('table').find_all('tr')
+def get_holiday_from_gov(year):
+    r = requests.get('https://www.gov.hk/en/about/abouthk/holiday/' + year + '.htm')
+    soup = BeautifulSoup(r.content.decode('utf-8'), 'lxml')
+    items = soup.find('table').find_all('tr')
 
-        holiday_date_list = []
-        half_day_mkt_date_list = []
+    holiday_date_list = []
+    half_day_mkt_date_list = []
 
-        for item in items[1:]:
-            tds = item.find_all('td')
-            holiday_date = tds[1].text + ' ' + year
-            holiday_date = datetime.datetime.strptime(holiday_date, '%d %B %Y').date()
-            holiday_name = tds[0].text.lower()
-            holiday_date_list.append(holiday_date)
-            if 'lunar new year' in holiday_name and 'the' not in holiday_name:
-                lin30_date = holiday_date - datetime.timedelta(days=1)
-                half_day_mkt_date_list.append(lin30_date)
+    for item in items[1:]:
+        tds = item.find_all('td')
+        holiday_date = tds[1].text + ' ' + year
+        holiday_date = datetime.datetime.strptime(holiday_date, '%d %B %Y').date()
+        holiday_name = tds[0].text.lower()
+        holiday_date_list.append(holiday_date)
+        if 'lunar new year' in holiday_name and 'the' not in holiday_name:
+            lin30_date = holiday_date - datetime.timedelta(days=1)
+            half_day_mkt_date_list.append(lin30_date)
 
-        xmax_eva_date = datetime.date(int(year), 12, 24)
-        if xmax_eva_date.weekday() < 5:
-            half_day_mkt_date_list.append(xmax_eva_date)
+    xmax_eva_date = datetime.date(int(year), 12, 24)
+    if xmax_eva_date.weekday() < 5:
+        half_day_mkt_date_list.append(xmax_eva_date)
 
-        xmax_eva_date = datetime.date(int(year), 12, 24)
-        if xmax_eva_date.weekday() < 5:
-            half_day_mkt_date_list.append(xmax_eva_date)
-        year_eva_date = datetime.date(int(year), 12, 31)
-        if year_eva_date.weekday() < 5:
-            half_day_mkt_date_list.append(year_eva_date)
+    xmax_eva_date = datetime.date(int(year), 12, 24)
+    if xmax_eva_date.weekday() < 5:
+        half_day_mkt_date_list.append(xmax_eva_date)
+    year_eva_date = datetime.date(int(year), 12, 31)
+    if year_eva_date.weekday() < 5:
+        half_day_mkt_date_list.append(year_eva_date)
 
-        return holiday_date_list, half_day_mkt_date_list
+    return holiday_date_list, half_day_mkt_date_list
 
-    def get_hk_holiday_and_expiry_date(self, start_year, end_year, format='int'):
-        holiday_date_list = []
-        half_day_mkt_date_list = []
+def get_hk_holiday_and_expiry_date(start_year, end_year, format='int'):
+    holiday_date_list = []
+    half_day_mkt_date_list = []
 
-        for i in range(end_year - start_year + 1):
-            this_year = end_year - i
-            this_year_str = str(this_year)
-            new_holiday_date_list, new_half_day_mkt_date_list = self.get_holiday_from_gov(this_year_str)
-            holiday_date_list = holiday_date_list + new_holiday_date_list
-            half_day_mkt_date_list = half_day_mkt_date_list + new_half_day_mkt_date_list
+    for i in range(end_year - start_year + 1):
+        this_year = end_year - i
+        this_year_str = str(this_year)
+        new_holiday_date_list, new_half_day_mkt_date_list = get_holiday_from_gov(this_year_str)
+        holiday_date_list = holiday_date_list + new_holiday_date_list
+        half_day_mkt_date_list = half_day_mkt_date_list + new_half_day_mkt_date_list
 
-        start_date = datetime.date(start_year, 1, 1)
-        end_date = datetime.date(end_year, 12, 31)
-        date_diff = (end_date - start_date).days
+    start_date = datetime.date(start_year, 1, 1)
+    end_date = datetime.date(end_year, 12, 31)
+    date_diff = (end_date - start_date).days
 
-        expiry_date_list = []
-        for i in range(date_diff):
-            date = end_date - datetime.timedelta(days=i)
-            last_date = date + datetime.timedelta(days=1)
-            if last_date.day == 1:
-                trading_days = 0
-                for j in range(7):
-                    test_date = date - datetime.timedelta(days=j)
-                    if test_date.weekday() < 5 and test_date not in holiday_date_list:
-                        trading_days += 1
-                    if trading_days == 2:
-                        if format == 'str':
-                            expiry_date = test_date.strftime('%Y%m%d')
-                        elif format == 'int':
-                            expiry_date = int(test_date.strftime('%Y%m%d'))
-                        elif format == 'datetime':
-                            expiry_date = test_date
-                        expiry_date_list.append(expiry_date)
-                        break
+    expiry_date_list = []
+    for i in range(date_diff):
+        date = end_date - datetime.timedelta(days=i)
+        last_date = date + datetime.timedelta(days=1)
+        if last_date.day == 1:
+            trading_days = 0
+            for j in range(7):
+                test_date = date - datetime.timedelta(days=j)
+                if test_date.weekday() < 5 and test_date not in holiday_date_list:
+                    trading_days += 1
+                if trading_days == 2:
+                    if format == 'str':
+                        expiry_date = test_date.strftime('%Y%m%d')
+                    elif format == 'int':
+                        expiry_date = int(test_date.strftime('%Y%m%d'))
+                    elif format == 'datetime':
+                        expiry_date = test_date
+                    expiry_date_list.append(expiry_date)
+                    break
 
-        holiday_list = []
-        for day in holiday_date_list:
-            if format == 'str':
-                holiday = day.strftime('%Y%m%d')
-            elif format == 'int':
-                holiday = int(day.strftime('%Y%m%d'))
-            elif format == 'datetime':
-                holiday = day
-            holiday_list.append(holiday)
+    holiday_list = []
+    for day in holiday_date_list:
+        if format == 'str':
+            holiday = day.strftime('%Y%m%d')
+        elif format == 'int':
+            holiday = int(day.strftime('%Y%m%d'))
+        elif format == 'datetime':
+            holiday = day
+        holiday_list.append(holiday)
 
-        half_day_mkt_list = []
-        for day in half_day_mkt_date_list:
-            if format == 'str':
-                holiday = day.strftime('%Y%m%d')
-            elif format == 'int':
-                holiday = int(day.strftime('%Y%m%d'))
-            elif format == 'datetime':
-                holiday = day
-            half_day_mkt_list.append(holiday)
+    half_day_mkt_list = []
+    for day in half_day_mkt_date_list:
+        if format == 'str':
+            holiday = day.strftime('%Y%m%d')
+        elif format == 'int':
+            holiday = int(day.strftime('%Y%m%d'))
+        elif format == 'datetime':
+            holiday = day
+        half_day_mkt_list.append(holiday)
+
+    dict1 = {'expiry_date': expiry_date_list, 'public_holiday': holiday_list, 'half_day_mkt': half_day_mkt_list}
+    return dict1
 
-        dict1 = {'expiry_date': expiry_date_list, 'public_holiday': holiday_list, 'half_day_mkt': half_day_mkt_list}
-        return dict1
